﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ATH
{
    public static class Config
    {
        public const char CSV_SEPARATOR = ';';
        public static char INI_SEPARATOR = '=';
        private static string COMMENT = "//";
        private const string configPath = @"./config.ini";
        private static Dictionary<string, string> options;

        static Config()
        {
            options = new Dictionary<string, string>();
            options.Add("SIMILARITY_MARGIN", 0.6f.ToString());
            options.Add("MIN_SIMILARITY_CLASS_SIZE", 3.ToString());
            options.Add("MIN_SIMILARITY_RATIO", 0.7f.ToString());
            options.Add("MAX_DEVIANT_MARGIN", 0.4f.ToString());
            options.Add("CSV_PATH", @"data.csv");
            //options.Add("WEIGHTED_SIMILARITY_SPACE_RENDER", false.ToString());
            options.Add("DECIMAL_PRECISION", 2.ToString());
            //options.Add("MATRIX_OUTPUT_FORMAT", "FIELD");
            options.Add("DUMP_MATRIX", true.ToString());
            options.Add("STOP_AFTER_DUMP", true.ToString());
        }

        public static void ParseConfig()
        {
            Console.WriteLine("Config initialization started...");
            Console.WriteLine(DateTime.Now.ToString("hh:mm:ss"));
            string[] options = Config.ReadFile();
            int i; float f; bool b;
            foreach (string o in options)
            {
                string key = o.Split(Config.INI_SEPARATOR)[0].Trim();
                string value = o.Split(Config.INI_SEPARATOR)[1].Trim();

                switch (key)
                {
                    case "SIMILARITY_MARGIN":
                    case "MIN_SIMILARITY_RATIO":
                    case "MAX_DEVIANT_MARGIN": string s = ReplaceDecSeparator(value); if (float.TryParse(s, out f)) Config.options[key] = f.ToString(); break;
                    case "MIN_SIMILARITY_CLASS_SIZE":
                    case "DECIMAL_PRECISION": if (Int32.TryParse(value, out i)) Config.options[key] = i.ToString(); break;
                    case "CSV_PATH": Config.options[key] = value.Trim(); break;
                    //case "MATRIX_OUTPUT_FORMAT": Config.options[key] = value.Trim(); break;
                    //case "WEIGHTED_SIMILARITY_SPACE_RENDER":
                    case "DUMP_MATRIX":
                    case "STOP_AFTER_DUMP": if (bool.TryParse(value, out b)) Config.options[key] = b.ToString(); break;
                }
            }
        }
        private static string[] ReadFile()
        {
            string line;
            List<string> lines = new List<string>();
            using (StreamReader file = new StreamReader(configPath))
            {
                while ((line = file.ReadLine()) != null)
                {
                    if(!line.StartsWith(COMMENT) && line.Trim() != String.Empty) lines.Add(line);
                }
                file.Close();
            }
            return lines.ToArray();
        }
        private static string ReplaceDecSeparator(string number)
        {
            string nDecimal = CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
            return number.Trim().Replace(",", nDecimal).Replace(".", nDecimal);
        }
        public static float FixDecSeparator(string number)
        {
            return float.Parse(Config.ReplaceDecSeparator(number));
        }

        public static float SIMILARITY_MARGIN
        {
            get { return float.Parse(Config.options["SIMILARITY_MARGIN"]); }
        }
        public static int MIN_SIMILARITY_CLASS_SIZE
        {
            get { return Int32.Parse(Config.options["MIN_SIMILARITY_CLASS_SIZE"]); }
        }
        public static float MIN_SIMILARITY_RATIO
        {
            get { return float.Parse(Config.options["MIN_SIMILARITY_RATIO"]); }
        }
        public static float MAX_DEVIANT_MARGIN
        {
            get { return float.Parse(Config.options["MAX_DEVIANT_MARGIN"]); }
        }
        public static string CSV_PATH
        {
            get { return Config.options["CSV_PATH"]; }
        }
        //public static bool WEIGHTED_SIMILARITY_SPACE_RENDER
        //{
        //    get { return bool.Parse(Config.options["WEIGHTED_SIMILARITY_SPACE_RENDER"]); }
        //}
        public static int DECIMAL_PRECISION
        {
            get { return Int32.Parse(Config.options["DECIMAL_PRECISION"]); }
        }
        //public static string MATRIX_OUTPUT_FORMAT
        //{
        //    get { return Config.options["MATRIX_OUTPUT_FORMAT"]; }
        //}
        public static bool DUMP_MATRIX
        {
            get { return bool.Parse(Config.options["DUMP_MATRIX"]); }
        }
        public static bool STOP_AFTER_DUMP
        {
            get { return bool.Parse(Config.options["STOP_AFTER_DUMP"]); }
        }
    }
}
