﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ATH
{
    public static class DataCSVHandler
    {
        private const int cacheInitSize = 17000000;

        private static readonly string outputPath;
        private static string[] lines;
        private static Int64 currentIndex;
        private static StringBuilder output = null;

        private static StreamWriter file;
        
        static DataCSVHandler()
        {
            DataCSVHandler.outputPath = @"SimFields_" + DateTime.Now.ToString("yyyyMMddhhmmss") + ".csv";
        }

        public static string[] ParseLine(string line)
        {
            return line.Split(Config.CSV_SEPARATOR);
        }
        /*
        public static NumberFormatInfo NumberFormat = new NumberFormatInfo
        {
            NumberGroupSeparator = String.Empty,
            NumberDecimalSeparator = ",",
            NumberDecimalDigits = Config.DECIMAL_PRECISION
        };
        */
        public static void ReadFile()
        {
            DataCSVHandler.Reset();
            string line;
            List<string> lines = new List<string>();
            try
            {
                StreamReader reader = new StreamReader(Config.CSV_PATH);
                while ((line = reader.ReadLine()) != null)
                {
                    lines.Add(line);
                }
                reader.Close();
                DataCSVHandler.lines = lines.ToArray();
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
            }
        }
        public static string[] GetRow()
        {
            if (DataCSVHandler.lines != null && DataCSVHandler.currentIndex < DataCSVHandler.lines.Count())
            {
                string[] row = DataCSVHandler.ParseLine(DataCSVHandler.lines[DataCSVHandler.currentIndex]);
                DataCSVHandler.currentIndex++;
                return row;
            }
            else return null;
        }
        
        public static string[] GetRow(int index)
        {
            return DataCSVHandler.ParseLine(DataCSVHandler.lines[index]);
        }

        public static void AppendOutput(Entity e1, Entity e2, float simValue)
        { 
            if(DataCSVHandler.output == null) DataCSVHandler.output = new StringBuilder(cacheInitSize);
            output.Append(e1.Id).Append(Config.CSV_SEPARATOR).Append(e1.Name).Append(Config.CSV_SEPARATOR);
            output.Append(e2.Id).Append(Config.CSV_SEPARATOR).Append(e2.Name).Append(Config.CSV_SEPARATOR);
            output.AppendLine(simValue.ToString());
        }

        public static void WriteCache()
        {
            file.Write(output.ToString());
            output.Clear();
            output = new StringBuilder(cacheInitSize);
        }
        public static void Write(Entity e1, Entity e2, float simValue)
        {

            file.Write(String.Format("{0}{5}{1}{5}{2}{5}{3}{5}{4}", e1.Id, e1.Name, e2.Id, e2.Name, simValue, Config.CSV_SEPARATOR));
        }

        public static void OpenOutputFile() 
        {
            if (!File.Exists(outputPath))
            {
                file = new StreamWriter(outputPath);
                //FileStream file = File.Create(outputPath);
            }
        }
        public static void CloseOutputFile()
        {
            file.Close();
        }
        public static void Reset() 
        {
            DataCSVHandler.lines = null;
            DataCSVHandler.ResetIndex();
        }
        public static void ResetIndex() 
        {
            DataCSVHandler.currentIndex = 1;
        }
        
        public static void Write(string classesString, string path)
        {
            Stopper.GetLastElapsed("Dumping Similarity Classes started. ");
            if (!File.Exists(path))
            {
                file = new StreamWriter(path);
                file.Write(classesString);
                file.Close();
            }
        }
        
        /*
        private static void WriteAverageSimilarities(IEnumerable<SimClass> classes)
        {
            using (var writer = new StreamWriter("reprezentansok.csv", false, Encoding.UTF8))
            {
                foreach (var simClass in classes)
                {
                    writer.WriteLine("{0}{1}{2}{1}{3}", simClass.Id, Config.CSV_SEPARATOR, simClass.Entities.Count, simClass.AverageSimilarityOfRepresentativeEntity);
                }
            }
        }

        private static void WriteAttributes(IEnumerable<SimClass> classes)
        {
            using (var writer = new StreamWriter("attributumok.csv", false, Encoding.UTF8))
            {
                foreach (var simClass in classes)
                {
                    foreach (var dataString in simClass.RelevantAttributesCsvData())
                    {
                        writer.WriteLine(dataString);
                    }
                }
            }
        }

        private static void WriteClasses(IEnumerable<SimClass> classes)
        {
            using (var writer = new StreamWriter("osztalyok.csv", false, Encoding.UTF8))
            {
                foreach (var simClass in classes)
                {
                    foreach (var dataString in simClass.EntitiesCsvData())
                    {
                        writer.WriteLine(dataString);
                    }
                }
            }
        }
         * */
        public static string[] Lines
        {
            get { return DataCSVHandler.lines; }
        }
        public static string Header
        {
            get { return lines[0]; }
        }
    }
}
