﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ATH
{
    public class SimClass
    {
        // ReSharper disable once RedundantDefaultFieldInitializer
        private static int _nextId = 0;

        private static readonly string classesOutPath;
        public static string ClassesOutPath { get { return classesOutPath; } }


        private readonly List<Entity> _entities = new List<Entity>();
        public List<Entity> Entities { get { return _entities; } }

        private Entity _representative;
        public Entity Representative { get { return _representative; } }

        private readonly List<RelevantAttribute> _relevantAttributeIndices = new List<RelevantAttribute>();
        public List<RelevantAttribute> RelevantAttributeIndices { get { return _relevantAttributeIndices; } }

        private readonly int _id;
        public int Id { get { return _id; } }
        static SimClass()
        {
            classesOutPath = @"SimClasses" + "_" + DateTime.Now.ToString("yyyyMMddhhmmss") + ".csv";
        }
        public double AverageSimilarityOfRepresentativeEntity
        {
            get
            {
                return _entities.Count > 1 ? _entities[0].RowSim / (Entities.Count - 1) : 0;
            }
        }

        public SimClass()
        {
            _id = _nextId++;
        }

        public void AddEntity(Entity entity)
        {
            _entities.Add(entity);
        }
        public void SetRepresentative(Entity entity)
        {
            _representative = entity;
            AddEntity(entity);
            entity.Classified = true;
        }

        public void DoAttributeReduction()
        {
            if (_entities.Count == 1)
            {
                return;
            }

            for (int i = 0; i < _entities[0].Attributes.Length; i++)
            {
                double average = _entities.ToList().Average(entity => entity.Attributes[i]);
                double deviation = _entities.ToList().Sum(entity => Math.Abs(average - entity.Attributes[i]));
                if (Math.Abs(average) < 0.001)
                {
                    continue;
                }
                double correctedDeviation = deviation / average / _entities.Count;
                if (correctedDeviation <= Config.MAX_DEVIANT_MARGIN)
                {
                    _relevantAttributeIndices.Add(new RelevantAttribute(i, average, correctedDeviation));
                }
            }
        }
        private float[][] GetAttributeQualifiers()
        {
            int length = this._entities[0].Attributes.Length;
            int memberCount = this._entities.Count;
            float[][] attrQualifier = new float[2][];
            attrQualifier[0] = new float[length];
            attrQualifier[1] = new float[length];
            for (int i = 0; i < length; i++)
            {
                float attrAvg = 0f;
                float attrDev = 0f;
                foreach (Entity e in this._entities)
                {
                    attrAvg += e.Attributes[i];
                }

                attrAvg = attrAvg / memberCount;
                attrQualifier[0][i] = (float)Math.Round(attrAvg, Config.DECIMAL_PRECISION);
                foreach (Entity e in this._entities)
                {
                    attrDev += Math.Abs(e.Attributes[i] - attrAvg);
                }

                float attrDevNom = attrAvg * memberCount;
                attrQualifier[1][i] = (float)Math.Round(((attrDevNom != 0) ? attrDev / attrDevNom : -1), Config.DECIMAL_PRECISION);
            }

            return attrQualifier;
        }
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("Class representative: ").Append(this.Representative.Name).Append("(ID: " + this.Representative.Id + ")").Append(", Class member count:").AppendLine(this._entities.Count.ToString());
            if (this._entities.Count > 0)
            {
                sb.Append("Deviation margin: ").AppendLine(Config.MAX_DEVIANT_MARGIN.ToString());
                sb.AppendLine("Qualified attributes table:");
                sb.AppendLine(DataCSVHandler.Header);
                float[][] attrQualifier = GetAttributeQualifiers();
                string avg = "AVG;";
                string dev = "DEV;";
                for (int i = 0; i < attrQualifier[0].Length; i++)
                {
                    avg = avg + attrQualifier[0][i] + Config.CSV_SEPARATOR;
                    float currDev = attrQualifier[1][i];
                    dev += ((currDev == -1) ? "####" : attrQualifier[1][i].ToString()) + Config.CSV_SEPARATOR;
                }
                sb.AppendLine(avg);
                sb.AppendLine(dev);
                sb.AppendLine();
                sb.AppendLine(DataCSVHandler.Header);
                foreach (Entity entity in this._entities)
                {
                    sb.Append(entity.ToString());
                }
            }
            return sb.ToString();
        }
        /*
        public IEnumerable<string> EntitiesCsvData()
        {
            // ReSharper disable once LoopCanBeConvertedToQuery
            foreach (var entity in _entities)
            {
                yield return String.Format("{1}{0}{2}", Config.CSV_SEPARATOR, _id, entity.BasicCsvString());
            }
        }

        public IEnumerable<string> RelevantAttributesCsvData()
        {
            // ReSharper disable once LoopCanBeConvertedToQuery
            foreach (var attribute in _relevantAttributeIndices)
            {
                yield return String.Format("{1}{0}{2}{0}{3}{0}{4}",
                    Config.CSV_SEPARATOR, _id, attribute.Id, attribute.Average.ToString(DataCSVHandler.NumberFormat), attribute.Deviation.ToString(DataCSVHandler.NumberFormat));
            }
        }
        */
    }
}
